"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var bt = __importStar(require("@babel/types"));
var recast_1 = __importDefault(require("recast"));
var babel_parser_1 = __importDefault(require("../babel-parser"));
var allowRE = /^(v-bind|:)/;
function propTemplateHandler(documentation, templateAst, options) {
    if (options.functional) {
        propsInAttributes(templateAst, documentation);
        propsInInterpolation(templateAst, documentation);
    }
}
exports.default = propTemplateHandler;
function propsInAttributes(templateAst, documentation) {
    var bindings = templateAst.attrsMap;
    var keys = Object.keys(bindings);
    for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
        var key = keys_1[_i];
        // only look at expressions
        if (allowRE.test(key)) {
            var expression = bindings[key];
            getPropsFromExpression(expression, documentation);
        }
    }
}
function propsInInterpolation(templateAst, documentation) {
    if (templateAst.children) {
        templateAst.children
            .filter(function (c) { return c.type === 2; })
            .forEach(function (expr) {
            getPropsFromExpression(expr.expression, documentation);
        });
    }
}
function getPropsFromExpression(expression, documentation) {
    var ast = babel_parser_1.default({ plugins: ['typescript'] }).parse(expression);
    recast_1.default.visit(ast.program, {
        visitMemberExpression: function (path) {
            var obj = path.node ? path.node.object : undefined;
            var propName = path.node ? path.node.property : undefined;
            if (obj &&
                propName &&
                bt.isIdentifier(obj) &&
                obj.name === 'props' &&
                bt.isIdentifier(propName)) {
                var pName = propName.name;
                var p = documentation.getPropDescriptor(pName);
                p.type = { name: 'undefined' };
            }
            return false;
        },
    });
}
//# sourceMappingURL=propHandler.js.map